@@ -5,16 +5,15 @@
 // or distributed except according to the terms contained in the COPYING file.
 //
 // Copyright (c) 2000 Shlomi Fish
-/*
-    pi_make_microsoft_freecell_board.c - Program to generate the initial
-    board of Microsoft Freecell or Freecell Pro for input to Freecell Solver.
 
-    Usage: pi-make-microsoft-freecell-board -t [board-number] | fc-solve
-
-    Based on the code by Jim Horne (who wrote the original Microsoft Freecell)
-    Based on code from the Microsoft C Run-Time Library.
-    Modified By Shlomi Fish, 2000
-*/
+// pi_make_microsoft_freecell_board.c - Program to generate the initial
+// board of Microsoft Freecell or Freecell Pro for input to Freecell Solver.
+//
+// Usage: pi-make-microsoft-freecell-board -t [board-number] | fc-solve
+//
+// Based on the code by Jim Horne (who wrote the original Microsoft Freecell)
+// Based on code from the Microsoft C Run-Time Library.
+// Modified By Shlomi Fish, 2000
 #include <stdio.h>
 #include <string.h>
 #include <stdbool.h>
@@ -33,10 +32,10 @@ int main(int argc, char *argv[])
     }
     const char *const deal_s = argv[arg];
 
-    const unsigned long long gamenumber =
-        ((arg < argc) ? (unsigned long long)atoll(deal_s) : 1);
-    const unsigned long long MAX = ((1ULL << 33) - 1);
-    const unsigned long long MIN = 1;
+    const fc_solve_ms_deal_idx_type gamenumber =
+        ((arg < argc) ? fcs_str2msdeal(deal_s) : 1);
+    const fc_solve_ms_deal_idx_type MAX = ((1ULL << 33) - 1);
+    const fc_solve_ms_deal_idx_type MIN = 1;
     if (gamenumber < MIN || gamenumber > MAX)
     {
         fprintf(stderr,
