@@ -24,11 +24,11 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <stdbool.h>
 
 #include "rb.h"
-
-#include "bool.h"
-#include "alloc_wrap.h"
+#include "rinutils/alloc_wrap.h"
+#include "rinutils/unused.h"
 
 #ifdef WITH_AVL_BALANCE_FIELD
 static inline signed char rb_get_color(struct rb_node *const node)
@@ -79,7 +79,7 @@ static inline enum rb_color rb_set_color(struct rb_node *const node, const enum
    and memory allocator |allocator|.
    Returns |NULL| if memory allocation failed. */
 struct rb_table *
-rb_create (rb_comparison_func *compare, void *param,
+rb_create (rb_comparison_func *compare, void *param GCC_UNUSED,
             meta_allocator *meta_alloc, void * * common_recycle_bin)
 {
   struct rb_table *tree;
@@ -94,7 +94,9 @@ rb_create (rb_comparison_func *compare, void *param,
 #define SET_TREE_AVL_ROOT(tree, val) rb_set_link(&(tree->rb_proto_root), 0, val)
   SET_TREE_AVL_ROOT(tree, NULL);
   tree->rb_compare = compare;
+#ifdef AVL_with_rb_param
   tree->rb_param = param;
+#endif
   fc_solve_compact_allocator_init(&(tree->dict_allocator), meta_alloc);
   tree->rb_recycle_bin = (struct rb_node * *)common_recycle_bin;
   tree->rb_count = 0;
@@ -113,7 +115,11 @@ rb_find (const struct rb_table *tree, const void *item)
   assert (tree != NULL && item != NULL);
   for (p = TREE_AVL_ROOT(tree); p != NULL; )
     {
-      int cmp = tree->rb_compare (item, NODE_DATA_PTR(p), tree->rb_param);
+      int cmp = tree->rb_compare (item, NODE_DATA_PTR(p)
+#ifdef AVL_with_rb_param
+          , tree->rb_param
+#endif
+          );
 
       if (cmp < 0)
         p = rb_process_link(p->rb_mylink[0]);
@@ -149,7 +155,11 @@ rb_probe (struct rb_table *tree, void *item)
       p != NULL;
       p = rb_process_link(p->rb_mylink[da[k - 1]]))
     {
-      int cmp = tree->rb_compare (item, NODE_DATA_PTR(p), tree->rb_param);
+      int cmp = tree->rb_compare (item, NODE_DATA_PTR(p)
+#ifdef AVL_with_rb_param
+                    , tree->rb_param
+#endif
+        );
       if (cmp == 0)
         return NODE_DATA_PTR(p);
 
@@ -302,12 +312,17 @@ rb_delete (struct rb_table *tree, const void *item)
   k = 0;
   p = (struct rb_node *) &tree->rb_proto_root;
   for (cmp = -1; cmp != 0;
-       cmp = tree->rb_compare (item, NODE_DATA_PTR(p), tree->rb_param))
+       cmp = tree->rb_compare (item, NODE_DATA_PTR(p)
+#ifdef AVL_with_rb_param
+
+           , tree->rb_param
+#endif
+           ))
     {
-      int dir = cmp > 0;
+      bool dir = cmp > 0;
 
       pa[k] = p;
-      da[k++] = dir;
+      da[k++] = (unsigned char)dir;
 
       p = rb_process_link(p->rb_mylink[dir]);
       if (p == NULL)
@@ -505,7 +520,9 @@ trav_refresh (struct rb_traverser *trav)
   if (trav->rb_node != NULL)
     {
       rb_comparison_func *cmp = trav->rb_table->rb_compare;
+#ifdef AVL_with_rb_param
       void *param = trav->rb_table->rb_param;
+#endif
       struct rb_node *node = trav->rb_node;
       struct rb_node *i;
 
@@ -516,7 +533,11 @@ trav_refresh (struct rb_traverser *trav)
           assert (i != NULL);
 
           trav->rb_stack[trav->rb_height++] = i;
-          i = rb_process_link(i->rb_mylink[cmp (NODE_DATA_PTR(node), NODE_DATA_PTR(i), param) > 0]);
+          i = rb_process_link(i->rb_mylink[cmp (NODE_DATA_PTR(node), NODE_DATA_PTR(i)
+#ifdef AVL_with_rb_param
+                  , param
+#endif
+                  ) > 0]);
         }
     }
 }
@@ -602,7 +623,11 @@ rb_t_find (struct rb_traverser *trav, struct rb_table *tree, void *item)
   trav->rb_generation = tree->rb_generation;
   for (p = TREE_AVL_ROOT(tree); p != NULL; p = q)
     {
-      int cmp = tree->rb_compare (item, NODE_DATA_PTR(p), tree->rb_param);
+      int cmp = tree->rb_compare (item, NODE_DATA_PTR(p)
+#ifdef AVL_with_rb_param
+          , tree->rb_param
+#endif
+          );
 
       if (cmp < 0)
         q = rb_process_link(p->rb_mylink[0]);
@@ -942,7 +967,13 @@ rb_destroy (struct rb_table *tree, rb_item_func *destroy)
       {
         q = rb_process_link(p->rb_mylink[1]);
         if (NODE_DATA_PTR(p) != NULL)
-          destroy (NODE_DATA_PTR(p), tree->rb_param);
+          destroy (NODE_DATA_PTR(p),
+#ifdef AVL_with_rb_param
+              tree->rb_param
+#else
+              NULL
+#endif
+              );
 #if 0
         tree->rb_alloc->libavl_free (tree->rb_alloc, p);
 #endif
