@@ -76,14 +76,13 @@ extern "C" {
 #include "pqueue.h"
 #include "meta_alloc.h"
 
-/* We need 2 chars per card - one for the column_idx and one
- * for the card_idx.
- *
- * We also need it times 13 for each of the ranks.
- *
- * We need (4*LOCAL_DECKS_NUM+1) slots to hold the cards plus a
- * (-1,-1) (= end) padding.
- * */
+// We need 2 chars per card - one for the column_idx and one
+// for the card_idx.
+//
+// We also need it times 13 for each of the ranks.
+//
+// We need (4*LOCAL_DECKS_NUM+1) slots to hold the cards plus a
+// (-1,-1) (= end) padding.
 #define FCS_POS_BY_RANK_WIDTH (MAX_NUM_DECKS << 3)
 
 // We don't keep track of kings
@@ -97,7 +96,7 @@ typedef struct
 {
     fcs_iters_int num_checked_states;
 #ifndef FCS_DISABLE_NUM_STORED_STATES
-    fcs_int_limit_t num_states_in_collection;
+    fcs_iters_int num_states_in_collection;
 #endif
 } fcs_stats;
 
@@ -120,10 +119,8 @@ typedef struct fcs_states_linked_list_item_struct
     struct fcs_states_linked_list_item_struct *next;
 } fcs_states_linked_list_item;
 
-/*
- * Declare these structures because they will be used within
- * fc_solve_instance, and they will contain a pointer to it.
- * */
+// Declare these structures because they will be used within
+// fc_solve_instance, and they will contain a pointer to it.
 struct fc_solve_hard_thread_struct;
 struct fc_solve_soft_thread_struct;
 struct fc_solve_instance_struct;
@@ -139,8 +136,10 @@ typedef void (*fc_solve_solve_for_state_move_func)(
 #define NUM_CHECKED_STATES                                                     \
     (HT_INSTANCE(hard_thread)->i__stats.num_checked_states)
 typedef struct fc_solve_instance_struct fcs_hard_thread;
+#if (defined(FCS_SINGLE_HARD_THREAD) && defined(FCS_WITH_MOVES))
 extern void fc_solve_init_soft_thread(fcs_hard_thread *const hard_thread,
     struct fc_solve_soft_thread_struct *const soft_thread);
+#endif
 #else
 #define HT_FIELD(hard_thread, field) (hard_thread)->field
 #define HT_INSTANCE(hard_thread) ((hard_thread)->instance)
@@ -154,9 +153,12 @@ extern bool fc_solve_check_and_add_state(
 #if (FCS_STATE_STORAGE == FCS_STATE_STORAGE_GLIB_HASH)
 extern guint fc_solve_hash_function(gconstpointer key);
 #endif
-#include "move_funcs_maps.h"
 
-/* HT_LOOP == hard threads' loop - macros to abstract it. */
+#ifndef FCS_ZERO_FREECELLS_MODE
+#include "move_funcs_maps.h"
+#endif
+
+// HT_LOOP == hard threads' loop - macros to abstract it.
 #ifdef FCS_SINGLE_HARD_THREAD
 
 #define HT_LOOP_START() fcs_hard_thread *const hard_thread = instance;
@@ -169,7 +171,7 @@ extern guint fc_solve_hash_function(gconstpointer key);
     for (; hard_thread < end_hard_thread; ++hard_thread)
 #endif
 
-/* ST_LOOP == soft threads' loop - macros to abstract it. */
+// ST_LOOP == soft threads' loop - macros to abstract it.
 #define ST_LOOP_START()                                                        \
     fcs_soft_thread *const ht_soft_threads =                                   \
         HT_FIELD(hard_thread, soft_threads);                                   \
@@ -201,6 +203,7 @@ typedef enum
     FCS_NO_SHUFFLING,
     FCS_RAND,
     FCS_WEIGHTING,
+    FCS_PERFORM_ALL_MOVE_FUNCS,
 } fcs_moves_group_kind;
 
 typedef union {
@@ -219,7 +222,9 @@ typedef struct
 typedef struct
 {
     uint_fast32_t num;
+#ifndef FCS_ZERO_FREECELLS_MODE
     fcs_moves_group *groups;
+#endif
 } fcs_moves_order;
 
 typedef struct
@@ -254,7 +259,7 @@ typedef struct
 #error Unknown FCS_RCS_CACHE_STORAGE
 #endif
     compact_allocator states_values_to_keys_allocator;
-    fcs_int_limit_t count_elements_in_cache, max_num_elements_in_cache;
+    fcs_iters_int count_elements_in_cache, max_num_elements_in_cache;
 
     fcs_cache_key_info *lowest_pri, *highest_pri, *recycle_bin;
 } fcs_lru_cache;
@@ -279,19 +284,15 @@ struct fc_solve_hard_thread_struct
     struct fc_solve_soft_thread_struct *soft_threads;
 
 #ifndef FCS_SINGLE_HARD_THREAD
-    /*
-     * The hard thread count of how many states he checked himself. The
-     * instance num_checked_states can be confusing because other threads
-     * modify it too.
-     *
-     * Thus, the soft thread switching should be done based on this variable
-     * */
+    // The hard thread count of how many states he checked himself. The
+    // instance num_checked_states can be misleading because other threads
+    // modify it too.
+    //
+    // Thus, the soft thread switching should be done based on this variable
     fcs_iters_int ht__num_checked_states;
 
 #endif
-    /*
-     * The maximal limit for num_checked_states.
-     * */
+    // The maximal limit for num_checked_states.
     fcs_iters_int ht__max_num_checked_states;
 
     // The index for the soft-thread that is currently processed
@@ -306,10 +307,8 @@ struct fc_solve_hard_thread_struct
     fcs_move_stack reusable_move_stack;
 #endif
 
-    /*
-     * This is a buffer used to temporarily store the stacks of the
-     * duplicated state.
-     * */
+    // This is a buffer used to temporarily store the stacks of the
+    // duplicated state.
     DECLARE_IND_BUF_T(indirect_stacks_buffer)
 
     size_t prelude_num_items;
@@ -333,8 +332,6 @@ struct fc_solve_hard_thread_struct
 #endif
 };
 
-/********************************************/
-
 typedef struct
 {
     int rating_with_index__idx;
@@ -348,8 +345,10 @@ typedef struct
     size_t move_func_list_idx;
     size_t current_state_index;
     size_t move_func_idx;
+#ifndef FCS_ZERO_FREECELLS_MODE
     size_t derived_states_random_indexes_max_size;
     rating_with_index *derived_states_random_indexes;
+#endif
     fcs__positions_by_rank positions_by_rank;
 } fcs_soft_dfs_stack_item;
 
@@ -383,12 +382,14 @@ struct fc_solve_soft_thread_struct
     // state-specific flags.
     fastest_type_for_num_soft_threads__unsigned id;
 
+#ifndef FCS_ZERO_FREECELLS_MODE
     // The moves' order indicates which move funcs to run.
     struct
     {
         size_t num;
         fcs_by_depth_moves_order *by_depth_moves;
     } by_depth_moves_order;
+#endif
 
     fcs_super_method_type super_method_type;
 
@@ -399,45 +400,39 @@ struct fc_solve_soft_thread_struct
             // The (temporary) max depth of the Soft-DFS scans)
             ssize_t dfs_max_depth;
 
-            /*
-             * Soft-DFS uses a stack of fcs_soft_dfs_stack_item s.
-             *
-             * derived_states_list - a list of states to be checked next.
-             * Not all of them will be checked because it is possible that
-             * future states already visited them.
-             *
-             * current_state_index - the index of the last checked state
-             * in depth i.
-             *
-             * move_func_list_idx - the index of the move list that is
-             * performed. FCS performs each move separately, so
-             * states_to_check and friends will not be overpopulated.
-             *
-             * num_vacant_stacks - the number of unoccpied stacks that
-             * correspond
-             * to solution_states.
-             *
-             * num_vacant_freecells - ditto for the freecells.
-             *
-             * */
+            // Soft-DFS uses a stack of fcs_soft_dfs_stack_item s.
+            //
+            // derived_states_list - a list of states to be checked next.
+            // Not all of them will be checked because it is possible that
+            // future states already visited them.
+            //
+            // current_state_index - the index of the last checked state
+            // in depth i.
+            //
+            // move_func_list_idx - the index of the move list that is
+            // performed. FCS performs each move separately, so
+            // states_to_check and friends will not be overpopulated.
+            //
+            // num_vacant_stacks - the number of unoccpied stacks that
+            // correspond
+            // to solution_states.
+            //
+            // num_vacant_freecells - ditto for the freecells.
             fcs_soft_dfs_stack_item *soft_dfs_info;
 
-            /* The depth of the DFS stacks */
+            // The depth of the DFS stacks
             ssize_t depth;
 
-            /*
-             * A pseudo-random number generator for use in the random-DFS
-             * scan
-             * */
             fcs_rand_gen rand_gen;
 
             // The initial seed of this random number generator
             fcs_rand_gen rand_seed;
 
-            /*
-             * The moves to be performed in a preprocessed form.
-             * */
+#ifndef FCS_ZERO_FREECELLS_MODE
+            // The moves to be performed in a preprocessed form.
             fcs_moves_by_depth_array moves_by_depth;
+#endif
+
         } soft_dfs;
         struct
         {
@@ -447,37 +442,25 @@ struct fc_solve_soft_thread_struct
             {
                 struct
                 {
-                    /*
-                     * A linked list that serves as the queue for the BFS
-                     * scan.
-                     * */
+                    // A linked list that serves as the queue for the BFS
+                    // scan.
                     fcs_states_linked_list_item *bfs_queue;
-                    /*
-                     * The last item in the linked list, so new items can be
-                     * added at
-                     * it, thus making it a queue.
-                     * */
+                    // The last item in the linked list, so new items can be
+                    // added at it, thus making it a queue.
                     fcs_states_linked_list_item *bfs_queue_last_item;
-                    /*
-                     * A linked list of items that were freed from
-                     * the queue and should be reused before allocating new
-                     * items.
-                     * */
+                    // A linked list of items that were freed from
+                    // the queue and should be reused before allocating new
+                    // items.
                     fcs_states_linked_list_item *recycle_bin;
                 } brfs;
                 struct
                 {
-                    /*
-                     * The priority queue of the BeFS scan
-                     * */
                     pri_queue pqueue;
                     fcs_state_weighting weighting;
                 } befs;
             } meth;
-            /*
-             * The first state to be checked by the scan. It is a kind of
-             * bootstrap for the algorithm.
-             * */
+            // The first state to be checked by the scan. It is a kind of
+            // bootstrap for the algorithm.
             fcs_collectible_state *first_state_to_check;
         } befs;
     } method_specific;
@@ -505,14 +488,10 @@ struct fc_solve_soft_thread_struct
 #endif
 
 #ifndef FCS_DISABLE_PATSOLVE
-    /*
-     * The patsolve soft_thread that is associated with this soft_thread.
-     * */
+    // The patsolve soft_thread that is associated with this soft_thread.
     struct fc_solve__patsolve_thread_struct *pats_scan;
 #endif
-    /*
-     * Differentiates between SOFT_DFS and RANDOM_DFS.
-     * */
+    // Differentiates between SOFT_DFS and RANDOM_DFS.
     bool master_to_randomize;
     bool is_befs;
 #ifdef FCS_WITH_MOVES
@@ -533,22 +512,20 @@ struct fc_solve_instance_struct
 
     fcs_stats i__stats;
 #ifndef FCS_WITHOUT_MAX_NUM_STATES
-    /*
-     * Limit for the maximal number of checked states.
-     * max_num_checked_states is useful because it can limit the amount of
-     * consumed memory (and time).
-     *
-     * This is the effective number that enables the process to work without
-     * checking if it's zero.
-     *
-     * Normally should be used instead.
-     * */
+    // Limit for the maximal number of checked states.
+    // max_num_checked_states is useful because it can limit the amount of
+    // consumed memory (and time).
+    //
+    // This is the effective number that enables the process to work without
+    // checking if it's zero.
+    //
+    // Normally should be used instead.
     fcs_iters_int effective_max_num_checked_states;
 #endif
 #ifndef FCS_DISABLE_NUM_STORED_STATES
-    fcs_int_limit_t effective_max_num_states_in_collection;
+    fcs_iters_int effective_max_num_states_in_collection;
 #ifndef FCS_WITHOUT_TRIM_MAX_STORED_STATES
-    fcs_int_limit_t effective_trim_states_in_collection_from;
+    fcs_iters_int effective_trim_states_in_collection_from;
 #endif
 #endif
     fcs_seq_cards_power_type initial_cards_under_sequences_value;
@@ -618,7 +595,7 @@ struct fc_solve_instance_struct
 // It gives a rough estimate of the memory occupied by the instance.
 #ifndef FCS_DISABLE_NUM_STORED_STATES
 #ifndef FCS_WITHOUT_TRIM_MAX_STORED_STATES
-    fcs_int_limit_t active_num_states_in_collection;
+    fcs_iters_int active_num_states_in_collection;
 #endif
 #endif
 
@@ -644,12 +621,10 @@ struct fc_solve_instance_struct
     // Soft-Threads.
     fcs_moves_order instance_moves_order;
 
-    /*
-     * A counter that determines how many of the hard threads that belong
-     * to this hard thread have already finished. If it becomes
-     * num_hard_threads the instance terminates.
-     * */
-    uint_fast32_t num_hard_threads_finished;
+    // Counts how many of the hard threads that belong
+    // to this instance have already finished. If it becomes
+    // num_hard_threads the instance terminates.
+    uint_fast32_t finished_hard_threads_count;
 
 #ifdef FCS_WITH_MOVES
     // The moves for the optimization scan, as specified by the user.
@@ -668,37 +643,30 @@ struct fc_solve_instance_struct
 #endif
 
 #ifndef FCS_WITHOUT_ITER_HANDLER
-    /*
-     * The debug_iter_output variables provide a programmer programmable way
-     * to debug the algorithm while it is running. This works well for DFS
-     * and Soft-DFS scans but at present support for BeFS and BFS is not
-     * too good, as its hard to tell which state came from which parent
-     * state.
-     *
-     * debug_iter_output_func is a pointer to the function that performs the
-     * debugging. If NULL, this feature is not used.
-     *
-     * debug_iter_output_context is a user-specified context for it, that
-     * may include data that is not included in the instance structure.
-     *
-     * This feature is used by the "-s" and "-i" flags of fc-solve-debug.
-     * */
+    // The debug_iter_output variables provide a programmable way
+    // to debug the algorithm while it is running. This works well for DFS
+    // and Soft-DFS scans but at present support for BeFS and BFS is not
+    // too good, as it's hard to tell which state came from which parent
+    // state.
+    //
+    // debug_iter_output_func is a pointer to the function that performs the
+    // debugging. If NULL, this feature is not used.
+    //
+    // debug_iter_output_context is a user-specified context for it, that
+    // may include data that is not included in the instance structure.
+    //
+    // This feature is used by the "-s" and "-i" flags of fc-solve-debug.
     instance_debug_iter_output_func debug_iter_output_func;
     void *debug_iter_output_context;
 #endif
 
-    /*
-     * The next ID to allocate for a soft-thread.
-     * */
     fastest_type_for_num_soft_threads__unsigned next_soft_thread_id;
 
-    /* This is the initial state */
+    // This is the initial state
     fcs_state_keyval_pair state_copy;
 
 #ifdef FCS_WITH_MOVES
-    /* This is the final state that the scan recommends to the
-     * interface
-     * */
+    // This is the final state that the scan recommends to the interface
     fcs_collectible_state *final_state;
 
     // A move stack that contains the moves leading to the solution.
@@ -735,7 +703,6 @@ struct fc_solve_instance_struct
 #define DFS_VAR(soft_thread, var) (soft_thread)->method_specific.soft_dfs.var
 #define BEFS_VAR(soft_thread, var)                                             \
     (soft_thread)->method_specific.befs.meth.befs.var
-/* M is Methods-common. */
 #define BEFS_M_VAR(soft_thread, var) (soft_thread)->method_specific.befs.var
 #define BRFS_VAR(soft_thread, var)                                             \
     (soft_thread)->method_specific.befs.meth.brfs.var
@@ -773,7 +740,7 @@ static inline int update_col_cards_under_sequences(
     const int sequences_are_built_by,
 #endif
     const fcs_const_cards_column col,
-    int d /* One less than cards_num of col. */
+    int d // One less than cards_num of col
 )
 {
     fcs_card this_card = fcs_col_get_card(col, d);
@@ -802,12 +769,6 @@ extern int fc_solve_compare_lru_cache_keys(const void *, const void *, void *);
 extern void fc_solve_soft_thread_init_befs_or_bfs(
     fcs_soft_thread *const soft_thread);
 
-extern void fc_solve_instance__init_hard_thread(
-#ifndef FCS_SINGLE_HARD_THREAD
-    fcs_instance *const instance,
-#endif
-    fcs_hard_thread *const hard_thread);
-
 extern void fc_solve_free_soft_thread_by_depth_move_array(
     fcs_soft_thread *const soft_thread);
 
@@ -815,16 +776,20 @@ static inline fcs_moves_order moves_order_dup(fcs_moves_order *const orig)
 {
     const_SLOT(num, orig);
     fcs_moves_order ret = (fcs_moves_order){.num = num,
-        .groups = memdup(orig->groups,
-            sizeof(orig->groups[0]) *
-                ((num & (~(MOVES_GROW_BY - 1))) + MOVES_GROW_BY))};
-
+#ifndef FCS_ZERO_FREECELLS_MODE
+        .groups = memdup(
+            orig->groups, sizeof(orig->groups[0]) *
+                              ((num & (~(MOVES_GROW_BY - 1))) + MOVES_GROW_BY))
+#endif
+    };
+#ifndef FCS_ZERO_FREECELLS_MODE
     for (size_t i = 0; i < num; ++i)
     {
         ret.groups[i].move_funcs = memdup(ret.groups[i].move_funcs,
             sizeof(ret.groups[i].move_funcs[0]) *
                 ((ret.groups[i].num & (~(MOVES_GROW_BY - 1))) + MOVES_GROW_BY));
     }
+#endif
 
     return ret;
 }
@@ -832,15 +797,15 @@ static inline fcs_moves_order moves_order_dup(fcs_moves_order *const orig)
 extern fcs_soft_thread *fc_solve_new_soft_thread(
     fcs_hard_thread *const hard_thread);
 
-/* This is the commmon code from fc_solve_instance__init_hard_thread() and
- * recycle_hard_thread() */
+// This is the commmon code from fc_solve_instance__init_hard_thread() and
+// recycle_hard_thread()
 static inline void fc_solve_reset_hard_thread(
     fcs_hard_thread *const hard_thread)
 {
 #ifndef FCS_SINGLE_HARD_THREAD
     HT_FIELD(hard_thread, ht__num_checked_states) = 0;
 #endif
-    HT_FIELD(hard_thread, ht__max_num_checked_states) = FCS_INT_LIMIT_MAX;
+    HT_FIELD(hard_thread, ht__max_num_checked_states) = FCS_ITERS_INT_MAX;
     HT_FIELD(hard_thread, num_soft_threads_finished) = 0;
 }
 
@@ -862,14 +827,9 @@ typedef enum
 extern void fc_solve_foreach_soft_thread(fcs_instance *const instance,
     const foreach_st_callback_choice callback_choice, void *const context);
 
-/*
-    This function is the last function that should be called in the
-    sequence of operations on instance, and it is meant for de-allocating
-    whatever memory was allocated by alloc_instance().
-  */
-
 static inline void moves_order__free(fcs_moves_order *moves_order)
 {
+#ifndef FCS_ZERO_FREECELLS_MODE
     const_SLOT(groups, moves_order);
     const_SLOT(num, moves_order);
     for (size_t group_idx = 0; group_idx < num; ++group_idx)
@@ -878,10 +838,10 @@ static inline void moves_order__free(fcs_moves_order *moves_order)
     }
     free(groups);
     moves_order->groups = NULL;
+#endif
     moves_order->num = 0;
 }
 
-/***********************************************************/
 #define MOVE_FUNC_ARGS                                                         \
     fcs_soft_thread *const soft_thread GCC_UNUSED,                             \
         fcs_kv_state raw_state_raw GCC_UNUSED,                                 \
@@ -900,8 +860,6 @@ static inline bool fcs_get_calc_real_depth(const fcs_instance *const instance)
 #ifdef FCS_WITH_MOVES
 extern void fc_solve_trace_solution(fcs_instance *const instance);
 #endif
-extern void fc_solve_finish_instance(fcs_instance *const instance);
-
 #ifdef __cplusplus
 }
 #endif
