@@ -15,6 +15,7 @@ extern "C" {
 #endif
 
 #include "gen_ms_boards__rand.h"
+#include "board_gen_lookup1.h"
 
 typedef size_t CARD;
 
@@ -30,29 +31,23 @@ static inline void card_to_string(char *const s, const CARD card)
     s[1] = card_to_string_suits[SUIT(card)];
 }
 
-#include "board_gen_lookup1.h"
-#ifdef FCS_GEN_BOARDS_WITH_EXTERNAL_API
-/* This is to settle gcc's -Wmissing-prototypes which complains about
- * missing prototypes for "extern" subroutines.
- *
- * It is not critical that it would be in the same place because the only
- * thing that uses this function is Python's ctypes (in the test files under
- * t/t/ ) which does not process the included C code. In the future, we may
- * have an external API in which case we'll devise a header for this
- * routine.
- *
- * */
-void DLLEXPORT fc_solve_get_board_l(
-    const unsigned long long deal_idx, char *const ret);
+static inline void get_board__setup_string(char *const ret)
+{
+    strcpy(ret, "XX XX XX XX XX XX XX\n"
+                "XX XX XX XX XX XX XX\n"
+                "XX XX XX XX XX XX XX\n"
+                "XX XX XX XX XX XX XX\n"
+                "XX XX XX XX XX XX\n"
+                "XX XX XX XX XX XX\n"
+                "XX XX XX XX XX XX\n"
+                "XX XX XX XX XX XX\n");
+}
 
-extern void DLLEXPORT fc_solve_get_board_l(
-#else
-static inline void get_board_l(
-#endif
+static inline void get_board_l__without_setup(
 #ifdef FCS_DEAL_ONLY_UP_TO_2G
-    const unsigned long long seedx
+    const fc_solve_ms_deal_idx_type seedx
 #else
-    const unsigned long long deal_idx
+    const fc_solve_ms_deal_idx_type deal_idx
 #endif
     ,
     char *const ret)
@@ -63,20 +58,9 @@ static inline void get_board_l(
     microsoft_rand seedx =
         microsoft_rand__calc_init_seedx((microsoft_rand)deal_idx);
 #endif
-    strcpy(ret, "XX XX XX XX XX XX XX\n"
-                "XX XX XX XX XX XX XX\n"
-                "XX XX XX XX XX XX XX\n"
-                "XX XX XX XX XX XX XX\n"
-                "XX XX XX XX XX XX\n"
-                "XX XX XX XX XX XX\n"
-                "XX XX XX XX XX XX\n"
-                "XX XX XX XX XX XX\n");
-
-    CARD deck[52]; /* deck of 52 unique cards */
-
-    /* shuffle cards */
-
-    for (size_t i = 0; i < 52; ++i) /* put unique card in each deck loc. */
+    // deck of 52 unique cards
+    CARD deck[52];
+    for (size_t i = 0; i < 52; ++i) // put unique card in each item
     {
         deck[i] = i;
     }
@@ -99,6 +83,41 @@ static inline void get_board_l(
     }
 }
 
+#ifdef FCS_GEN_BOARDS_WITH_EXTERNAL_API
+// This is to settle gcc's -Wmissing-prototypes which complains about
+// missing prototypes for "extern" subroutines.
+//
+// It is not critical that it would be in the same place because the only
+// thing that uses this function is Python's ctypes (in the test files under
+// t/t/ ) which does not process the included C code. In the future, we may
+// have an external API in which case we'll devise a header for this
+// routine.
+void DLLEXPORT fc_solve_get_board_l(
+    const fc_solve_ms_deal_idx_type deal_idx, char *const ret);
+#endif
+
+#ifdef FCS_GEN_BOARDS_WITH_EXTERNAL_API
+// This is to settle gcc's -Wmissing-prototypes which complains about
+// missing prototypes for "extern" subroutines.
+//
+// It is not critical that it would be in the same place because the only
+// thing that uses this function is Python's ctypes (in the test files under
+// t/t/ ) which does not process the included C code. In the future, we may
+// have an external API in which case we'll devise a header for this
+// routine.
+void DLLEXPORT fc_solve_get_board_l(
+    const fc_solve_ms_deal_idx_type deal_idx, char *const ret);
+
+extern void DLLEXPORT fc_solve_get_board_l(
+#else
+static inline void get_board_l(
+#endif
+    const fc_solve_ms_deal_idx_type seedx, char *const ret)
+{
+    get_board__setup_string(ret);
+    get_board_l__without_setup(seedx, ret);
+}
+
 #ifndef FCS_GEN_BOARDS_WITH_EXTERNAL_API
 static inline void get_board(unsigned long deal_idx, char *ret)
 {
