@@ -22,7 +22,7 @@ my $use_prove = $ENV{FCS_USE_TEST_RUN} ? 0 : 1;
 my $num_jobs  = $ENV{TEST_JOBS};
 my $KEY       = 'FC_SOLVE__TESTS_RERUNS_CACHE_DATA_DIR';
 my $cache     = CHI->new(
-    driver => 'File',
+    driver   => 'File',
     root_dir =>
         ( $ENV{$KEY} || ( ( $ENV{TMPDIR} || '/tmp' ) . '/fc-solve1temp' ) )
 );
@@ -76,17 +76,29 @@ sub run_tests
 my $tests_glob = "*.{t.exe,py,t}";
 my $exclude_re_s;
 my $force_rebuild = delete( $ENV{REBUILD} );
+my $tests_dir;
 
 my @execute;
+my $should_skip  = 0;
+my $custom_tests = 0;
 GetOptions(
-    '--exclude-re=s' => \$exclude_re_s,
-    '--execute|e=s'  => \@execute,
-    '--exit0!'       => \$exit_success,
-    '--glob=s'       => sub { $tests_glob = $_[1]; $glob_was_set = 1; },
-    '--prove!'       => \$use_prove,
-    '--rebuild!'     => \$force_rebuild,
-    '--jobs|j=n'     => \$num_jobs,
+    '--custom-tests-suite!' => \$custom_tests,
+    '--exclude-re=s'        => \$exclude_re_s,
+    '--execute|e=s'         => \@execute,
+    '--exit0!'              => \$exit_success,
+    '--glob=s'              => sub { $tests_glob = $_[1]; $glob_was_set = 1; },
+    '--jobs|j=n'            => \$num_jobs,
+    '--prove!'              => \$use_prove,
+    '--rebuild!'            => \$force_rebuild,
+    '--should-skip-tests!'  => \$should_skip,
+    '--tests-dir=s'         => \$tests_dir,
 ) or die "Wrong opts - $!";
+
+if ( defined $tests_dir )
+{
+    $tests_dir = path($tests_dir)->absolute;
+}
+
 my %binaries;
 %progs =
     map { $_ => +{ binaries => {}, } }
@@ -94,7 +106,7 @@ my %binaries;
     FC_Solve::Paths::Base::exe_fn( $_->{args}->{prog} // die %{ $_->{args} } )
     } values %{ FC_Solve::Test::Valgrind::Data->get_hash };
 
-use Digest::SHA ();
+use Digest ();
 if ( $FC_Solve::Paths::Base::IS_WIN || $force_rebuild )
 {
     ++$rerun;
@@ -111,7 +123,7 @@ else
             $progs{$prog}{binaries}{$bin} //= (
                 $binaries{$bin} //= do
                 {
-                    Digest::SHA->new(256)->addfile($bin)->b64digest;
+                    Digest->new('SHA-256')->addfile($bin)->b64digest;
                 }
             );
         }
@@ -134,10 +146,37 @@ sub myglob
     local $ENV{FCS_PATH}                = $fcs_path;
     local $ENV{FCS_SRC_PATH}            = $abs_bindir;
     local $ENV{PYTHONDONTWRITEBYTECODE} = '1';
-
-    local $ENV{FCS_TEST_TAGS} = join ' ',
-        sort { $a cmp $b }
-        ( path('.')->child(qw/t TAGS.txt/)->slurp_utf8 =~ /([a-zA-Z_0-9]+)/g );
+    my %TEST_TAGS = (
+        map { $_ => 1 } (
+            path('.')->child(qw/t TAGS.txt/)->slurp_utf8 =~ /([a-zA-Z_0-9]+)/g
+        )
+    );
+    my $tests_may_fail = ( exists $TEST_TAGS{'tests_may_fail'} );
+    if ( !$custom_tests )
+    {
+        if ($should_skip)
+        {
+            if ($tests_may_fail)
+            {
+                print "Yes, the tests should be skipped.\n";
+                exit(0);
+            }
+            else
+            {
+                print "No, the tests ought to succeed in this configuration.\n";
+                exit(1);
+            }
+        }
+        elsif ($tests_may_fail)
+        {
+            die "The tests should be skipped.";
+        }
+    }
+    else
+    {
+        delete $TEST_TAGS{'tests_may_fail'};
+    }
+    local $ENV{FCS_TEST_TAGS} = join ' ', sort { $a cmp $b } keys %TEST_TAGS;
 
     my $preset_dest     = path('.')->child( "t", "Presets" );
     my $preset_src      = $abs_bindir->child("Presets");
@@ -234,13 +273,15 @@ sub myglob
             || ( basename($a) cmp basename($b) )
             || ( $a cmp $b )
         } (
-        myglob('t'),
-        myglob('.'),
-        (
-              ( $fcs_path ne $abs_bindir )
-            ? ( myglob("$abs_bindir/t/t") )
-            : ()
-        ),
+        defined($tests_dir) ? ( myglob($tests_dir) )
+        : (
+            myglob('t'),
+            myglob('.'),
+            (
+                  ( $fcs_path ne $abs_bindir ) ? ( myglob("$abs_bindir/t/t") )
+                : ()
+            ),
+        )
         );
 
     if ( defined($exclude_re_s) )
@@ -276,7 +317,10 @@ EOF
     }
     elsif (@execute)
     {
-        system(@execute);
+        if ( system(@execute) )
+        {
+            die "execute failed";
+        }
     }
     else
     {
