@@ -18,18 +18,17 @@ static inline int fcs_stack_compare(const void *s1, const void *s2)
 #undef GET_CARD
 #endif
 
+#define GET_STACK(c) fcs_state_get_col(*state_key, (c))
 #ifdef COMPACT_STATES
 
-#define DECLARE_TEMP_STACK() char temp_stack[MAX_NUM_CARDS_IN_A_STACK + 1]
+#define DECLARE_TEMP_STACK() char temp_stack[FCS_CARDS_COL_WIDTH]
 #define STACK_COMPARE(a, b) (fcs_stack_compare((a), (b)))
-#define GET_STACK(c) (state_key->data + (c) * (MAX_NUM_CARDS_IN_A_STACK + 1))
-#define COPY_STACK(d, s) (memcpy(d, s, (MAX_NUM_CARDS_IN_A_STACK + 1)))
+#define COPY_STACK(d, s) (memcpy(d, s, FCS_CARDS_COL_WIDTH))
 
 #elif defined(INDIRECT_STACK_STATES)
 
 #define DECLARE_TEMP_STACK() fcs_card *temp_stack
 #define STACK_COMPARE(a, b) (fc_solve_stack_compare_for_canonize(a, b))
-#define GET_STACK(c) (state_key->columns[c])
 #define COPY_STACK(d, s) (d = s)
 
 #endif
@@ -41,7 +40,7 @@ void fc_solve_canonize_state(
     fcs_state *const ptr_state_key FREECELLS_AND_STACKS_ARGS())
 {
 #define state_key (ptr_state_key)
-    /* Insertion-sort the columns */
+    // Insertion-sort the columns
 
     for (size_t b = 1; b < STACKS_NUM__VAL; b++)
     {
